from .attachments import FILE_MAX_SIZE, CLIENT_MAX_ATTACHMENTS
from .requests import ENDPOINT, SECONDS_BEFORE_TIMEOUT

__all__ = [
    "FILE_MAX_SIZE",
    "CLIENT_MAX_ATTACHMENTS",
    "ENDPOINT",
    "SECONDS_BEFORE_TIMEOUT",
]
